'use strict';

const webpack = require('webpack');
const packageJSON = require('../../package.json');
const legoConfig = packageJSON.legoconfig;
const f2eCiConfig = require('../../f2eci.json');

const env = f2eCiConfig.env;

const resourceResolver = require('./resourceresolver.js');

function getEntries() {
    const entry = {};
    entry[legoConfig.vuechunkname] = ['lego-supportor/lib/vendors/vue.js'];
    entry[legoConfig.reactchunkname] = ['lego-supportor/lib/vendors/react.js'];
    return entry;
}

const plugins = [];
if (env === 'product') {
    plugins.push(new webpack.DefinePlugin({
        'process.env': {
            NODE_ENV: JSON.stringify('production')
        }
    }));
    plugins.push(new webpack.optimize.UglifyJsPlugin({
        compress: {
            warnings: false
        }
    }));
}

module.exports = {
    entry: getEntries(),
    output: {
        filename: '[name].js',
        path: resourceResolver.WEBPACK_DIST,
        publicPath: resourceResolver.PUBLIC_PATH,
        chunkFilename: '[name].[chunkhash].js'
    },
    cache: true,
    resolve: {
        // 添加后缀名顺序
        extensions: ['.js', '.jsx', '.vue', '.less']
    },
    target: 'web',
    module: {
        rules: [{
            test: /\.(js)$/,
            exclude: /node_modules\/(?!(@gfe|lego))/,
            use: [{
                loader: 'babel-loader',
                options: {
                    presets: ['es2015', 'stage-0'],
                    cacheDirectory: true
                }
            }]
        }]
    },
    plugins
};
