'use strict';

// 设置lego的环境参数
process.env.LEGO_ENV = 'dev';
process.env.NODE_ENV = 'development';

const webpack = require('webpack');
const ProgressBarPlugin = require('progress-bar-webpack-plugin');
const serverConfig = require('./webpack/server/widget.config.js');

const path = require('path');
const co = require('co');
const accountFilter = require('@dp/account-filter');
const app = require('@dp/node-server');
const Lego = require('@gfe/lego');
const fetch = require('@gfe/m-isomorphic-fetch');

/**
 * 初始化服务端编译
 *
 * @return {Promise}
 */
function initServerCompilePromise() {
    serverConfig.plugins = serverConfig.plugins || [];
    serverConfig.plugins.push(new ProgressBarPlugin());
    const serverCompiler = webpack(serverConfig);
    return new Promise((resolve, reject) => {
        serverCompiler.run((err, stats) => {
            stats = stats.toJson();
            stats.errors.forEach((error) => {
                console.error(error);
            });
            stats.warnings.forEach((error) => {
                console.warn(error);
            });
            if (err) {
                reject(err);
                return;
            }
            console.log('server compile done.');
            watchServerCompile(serverCompiler);
            resolve('done');
        });
    });
}

/**
 * 开启server端的watch监听
 *
 * @param {Object} serverCompiler - 对应的webpack编译器
 */
function watchServerCompile(serverCompiler) {
    serverCompiler.watch({}, (err, stats) => {
        stats = stats.toJson();
        stats.errors.forEach((error) => {
            console.error(error);
        });
        stats.warnings.forEach((error) => {
            console.warn(error);
        });
        if (err) throw err;
        // 提示lego删除自己cache的内容
        Lego.updateWidgets();
    });
}

function startApp() {
    co(function*() {
        // 初始化编译
        yield initServerCompilePromise();

        yield app.init({
            appRoot: __dirname,
            urlCare: [/^\/fitness\/node\/html/]
        });
        /**
         *
         * app.use(function*(next){})
         *
         * */
        yield accountFilter.init();
        // 挂载fetch的中间件

        app.use(fetch.init({
            onPrepareUrl(relativePath, options, context) {
                let url = fetch.defaultUrlRewrite(relativePath, options, context);
                url = require('@dp/parrot-mocker').wrapUrl(url, {
                    pageUrl: context.url,
                    cookie: context.header.cookie
                });
                return url;
            },
            lionFallbackKey: 'node-fitness-web.fallback.domain',
            fallbackUrl: '/fitness/node/widgetsender.jsonp'
        }));

        app.use(accountFilter.middleware);
        app.use(Lego.init({
            actionDir: path.resolve(__dirname, './action'),
            actionNameRegex: /\.(action)$/,
            widgetDir: path.resolve(__dirname, './serverdist'),
            widgetNameRegex: /\.(js)$/,
            vendors: {
                react: '<%- PkgName %>/static/react.js'
            }
        }));

        app.start();
    }).catch((e) => {
        console.log(e.stack);
        app.context.logger.error(e.stack);
        process.exit(1);
    });
}

startApp();
